package com.joelhelkala.watcherGui.Node;

import java.awt.LayoutManager;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import com.joelhelkala.watcherGui.Node.NodeData.NodeData;

public class Node {
	private String description;
	private String location;
	private Float latitude;
	private Float longitude;
	private List<NodeData> data;
	
	public Node(String description, Float latitude, Float longitude, String location) {
		this.description = description;
		this.latitude = latitude;
		this.longitude = longitude;
		this.location = location;
		this.data = new ArrayList<NodeData>();
	}
	
	public Node(JSONArray arr) {
		if(arr.length() == 0) {
			this.description = "";
			this.location = "";
			this.latitude = 0f;
			this.longitude = 0f;
			this.data = new ArrayList<NodeData>();
		} else {			
			JSONObject node = arr.getJSONObject(0);
			JSONObject parent = node.getJSONObject("parent");
			this.description = parent.getString("description");
			this.location = parent.getString("location");
			this.latitude = parent.getFloat("latitude");
			this.longitude = parent.getFloat("longitude");
			this.data = new ArrayList<NodeData>();
			
			for(int i = 0; i < arr.length(); i++) {
				JSONObject measurements = arr.getJSONObject(i);
				LocalDateTime measured_at = LocalDateTime.parse(measurements.getString("measured_at"));
				Integer temperature = measurements.getInt("temperature");
				Integer humidity = measurements.getInt("humidity");
				Integer luminosity = measurements.getInt("luminosity");
				this.data.add(new NodeData(measured_at, temperature, humidity, luminosity));
			}
		}
	}

	// Gets the temperature reading of the most recent measurement
	public Integer getRecentTemperature() {
		NodeData data = findRecentData();
		if(data == null) return 0;
		return data.getTemperature();
	}

	public Integer getRecentHumidity() {
		NodeData data = findRecentData(); 
		if(data == null) return 0;
		return data.getHumidity();
	}
	
	
	public LocalDateTime getRecentDate() {
		NodeData data = findRecentData();
		if(data == null) return null;
		return data.getTime();
	}
	
	private NodeData findRecentData() {
		if(data.size() == 0) return null;
		NodeData newest = data.get(0);
		for(int i = 1; i < data.size(); i++) {
			if(data.get(i).getTime().isAfter(newest.getTime())) newest = data.get(i);
		}
		return newest;
	}

	public String getLocation() {
		return location;
	}
	
}
